#----------------------------------------------------------------------
# TITLE:
#	manrules.tcl
#
# AUTHOR:
#	Andreas Kupries
#
# DESCRIPTION:
#	Rules and prcedures to create manpages from annotated text.
#----------------------------------------------------------------------

proc setx {v string} {
    upvar $v _
    set _ $string
    return
}

proc appendx {v string} {
    upvar $v _
    append _ $string
    return
}

proc begin_hook {} {
    # Called before the first output.
}

proc end_hook {} {
    # Called after the last output.
}

proc comment {text} {
    return "'\\\" $text"
}

proc heading {manpage section description} {
    set    heading ""
    append heading "[comment {}]\n"
    append heading "[comment {Copyright (c) 2000 Andreas Kupries}]\n"
    append heading "[comment {}]\n"
    append heading "[comment {See the file "license.terms" for information on usage and redistribution}]\n"
    append heading "[comment {of this file, and for a DISCLAIMER OF ALL WARRANTIES.}]\n"
    append heading "[comment {}]\n"
    append heading "[comment "SCCS: @(#) $manpage.$section"]\n"
    append heading "[comment {}]\n"
    append heading ".so man.macros\n"
    append heading ".TH \"$manpage\" n 8.2 Tcl \"Tcl Built-In Commands\"\n"
    append heading ".BS\n"
    append heading "[comment {Note:  do not modify the .SH NAME line immediately below!}]\n"
    append heading ".SH NAME\n"
    append heading "$manpage \\- $description"
    return $heading
}

proc headend    {}     {return .BE}
proc section    {name} {return ".SH \"$name\""}
proc bold       {}     {return \\fB}
proc underline  {}     {return \\fI}
proc ul         {}     {return \\fI}
proc reset      {}     {return \\fR}
proc para       {}     {return .PP}
proc bullet     {text} {return ".TP\n$text"}
proc bulletn    {n text} {return ".TP $n\n$text"}
proc verbatimS  {}     {return .CS}
proc verbatimE  {}     {return .CE}
proc numitem    {i}    {return ".IP \[$i\]"}
proc vtab       {}     {return .sp}

proc boldt      {text} {return "[bold]$text[reset]"}
proc underlinet {text} {return "[underline]$text[reset]"}
proc ult        {text} {return "[underline]$text[reset]"}
proc changebar-on {text} {return ".VS $text"}
proc changebar-end {}     {return .VE}

# -------------------------------------------------------------------
# Higher level procedures....

proc examples {text} {
    set text [string trim $text]
    return "[section EXAMPLES]\n[verbatimS]\n$text\n[verbatimE]"
}

proc keywords {args} {
    return "[section KEYWORDS]\n[join $args ", "]"
}

proc arg {text} {ult   $text}
proc cmd {text} {boldt $text}

proc synopsis {text} {
    return "[section SYNOPSIS]\n[bold] $text"
}

proc synopsis_n {list} {
    set hdr [section SYNOPSIS]
    foreach text $list {
	append hdr "\n[bold] $text"
    }
    return $hdr
}


proc header {command section description arglist} {
    set     hdr [heading  $command $section $description]\n
    append  hdr [synopsis "$command $arglist"]\n
    append  hdr [headend]
    return $hdr
}

proc headers {title section description args} {
    set     hdr [heading  $title $section $description]\n
    append  hdr [synopsis_n $args]\n
    append  hdr [headend]
    return $hdr
}

proc description {text} {
    return "[section DESCRIPTION]\n[para]\n[string trim $text]"
}
